#!/bin/ksh
#DEBUG=TRUE

#####################################################################
#
# Licensed Materials - Property of IBM
#
# All Rights Reserved
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#####################################################################
#
# PROGRAM NAME: install_update.sh
#
# PURPOSE: install or uninstall patches or fix packs for
#   Secureway Directory Server 3.2.1
#   Secureway Directory Server 3.2.2
#   IBM Directory Server 4.1
#   IBM Directory Server 5.1
#   IBM Tivoli Directory Server 5.2
#
# PLATFORMS: AIX Solaris HP-UX Linux-x86 Linux-390 Linux-i/p
#
#####################################################################
if [[ ${DEBUG} = TRUE ]]; then
    set -x
fi


#####################################################################
#
# Init_Platform
#
# Note: Should only be called from Init_Patch_Data()
#
# Initialize the "PLATFORM_*" variables to one of:
#   PLATFORM_ID   = { A   | H     | S       | T         | P         | L         | x       }
#   PLATFORM_NAME = { AIX | HP-UX | Solaris | Linux-390 | Linux-i/p | Linux-x86 | unknown }
# Return 0 iff ( PLATFORM_ID != "x" && PLATFORM_NAME != "unknown" )
#

PLATFORM_ID="x"
PLATFORM_NAME="unknown"

Init_Platform()
{
    if [[ ${DEBUG} = TRUE ]]; then
        set -x
    fi
    
    os=$(uname)
    case ${os} in
    AIX )
        PLATFORM_ID="A"
        PLATFORM_NAME="AIX" ;;
    HP-UX )
        PLATFORM_ID="H"
        PLATFORM_NAME="HP-UX" ;;
    SunOS )
        PLATFORM_ID="S"
        PLATFORM_NAME="Solaris" ;;
    Linux )
        mach=$(uname -m)
        case ${mach} in
        s390 )
            PLATFORM_ID="T"
            PLATFORM_NAME="Linux-390" ;;
        ppc64 )
            PLATFORM_ID="P"
            PLATFORM_NAME="Linux-i/p" ;;
        *86 )
            PLATFORM_ID="L"
            PLATFORM_NAME="Linux-x86" ;;
        * )
            print "Unknown Linux machine type: \"${mach}\""
            return 1 ;;
        esac ;;
    * )
        print "Unknown OS: \"${os}\""
        return 1 ;;
    esac
    
    return 0
}


#####################################################################
#
# Init_Patch_Data
#
# Use script name & directory to intialize script variables 
#
#                  Example 1                          Example 2
# Input            ---------                          ---------
#   script         FP322S-02/install_update.sh        P510L-04G/uninstall_update.sh
# Output           ---------                          ---------
#   ACTION         install                            uninstall
#   PATCH_NAME     FP322S-02                          P510L-04G
#   PLATFORM_ID    S                                  L
#   PLATFORM_NAME  Solaris                            Linux-x86
#   PATCH_TYPE     fix pack                           patch
#   PATCH_VER      322                                510
#   PATCH_FP       02                                 04
#   PATCH_LTR      ""                                 L
#   IDS_NAME       Secureway Directory Server v3.2.2  IBM Directory Server v5.1
#   IDS_EXE        slapd                              ibmslapd
#   LDAPHome       /opt                               /usr/ldap
#   PATCH_ID_DIR   IBMldapc/bin                       bin
#   ID_FILE        IBMldapc/bin/FP322S-02.txt         bin/P510L-04G.txt
#   COMPRESS       compress                           gzip
#   COMP_EXT       Z                                  gz

# Are we installing or uninstalling?
if [[ $(basename ${0}) = un* ]]; then
    ACTION="uninstall"
else 
    ACTION="install"
fi
    
# Where are we (un)installing from?
PATCH_DIR=$(dirname ${0})
cd ${PATCH_DIR}
PATCH_DIR=$(pwd)

PATCH_NAME=""   # What are we (un)installing? E.g.: "P510L-04G"
PATCH_TYPE=""   # one of { "patch" "fix pack" }
PATCH_VER=""    # one of { 321 322 410 510 520 }
PATCH_FP=""     # fix pack level, e.g.: "00", "01", "02"... etc. 
PATCH_LTR=""    # A-Z or "" if a fix pack
IDS_NAME=""     # Full product name for ${PATCH_VER}
IDS_EXE=""      # name of server: slapd or ibmslapd
LDAPHome=""     # install path
PATCH_ID_DIR="" # Relative path to patch ID file
ID_FILE=""      # relative path to patch-id file, e.g.: "IBMldapc/bin/P322S-04G.txt"
COMPRESS=""     # compression program: "compress" or "gzip"
COMP_EXT=""     # compressed archive extension: "Z" or ".gz"

Init_Patch_Data()
{
    if [[ ${DEBUG} = TRUE ]]; then
        set -x
    fi
    
    # What are we (un)installing?
    PATCH_NAME=$(basename ${PATCH_DIR})
    # ${PATCH_NAME} must be exactly 9 characters
    if [[ 9 != ${#PATCH_NAME} ]]; then
        print "${PATCH_NAME} is not a valid patch identifier."
        return 1
    fi

    # What platform are we on?
    Init_Platform
    if [[ $? -ne 0 ]]; then
        return 1
    fi

    # Initialize platform specific defaults...
    case ${PLATFORM_ID} in
    A )
        LDAPHome="/usr/ldap"
        PATCH_ID_DIR="bin"
        COMPRESS="compress"    
        COMP_EXT="Z" ;;
    H )
        LDAPHome="/usr/IBMldap"
        PATCH_ID_DIR="bin"
        COMPRESS="compress"    
        COMP_EXT="Z" ;;
    S )
        LDAPHome="/opt"
        PATCH_ID_DIR="IBMldapc/bin"
        COMPRESS="compress"    
        COMP_EXT="Z" ;;
    L )
        LDAPHome="/usr/ldap"
        PATCH_ID_DIR="bin"
        COMPRESS="gzip"    
        COMP_EXT="gz" ;;
    P )
        LDAPHome="/usr/ldap"
        PATCH_ID_DIR="bin"
        COMPRESS="gzip"    
        COMP_EXT="gz" ;;
    T )
        LDAPHome="/usr/ldap"
        PATCH_ID_DIR="bin"
        COMPRESS="gzip"    
        COMP_EXT="gz" ;;
    * )
        print "Invalid platform ID: ${1}"
        return 1
    esac

    # Parse the PATCH_NAME (e.g.: FP410A-02 or P520P-03K) into it's sub-fields:
    case ${PATCH_NAME} in
    FP* )
        PATCH_TYPE="fix pack"
        PATCH_VER=$(echo ${PATCH_NAME} | cut -c3-5)
        PATCH_PLATFORM=$(echo ${PATCH_NAME} | cut -c6)
        HYPHEN=$(echo ${PATCH_NAME} | cut -c7)
        PATCH_FP=$(echo ${PATCH_NAME} | cut -c8-9) 
        PATCH_LTR="" ;;
    P* )
        PATCH_TYPE="patch"
        PATCH_VER=$(echo ${PATCH_NAME} | cut -c2-4)
        PATCH_PLATFORM=$(echo ${PATCH_NAME} | cut -c5)
        HYPHEN=$(echo ${PATCH_NAME} | cut -c6)
        PATCH_FP=$(echo ${PATCH_NAME} | cut -c7-8) 
        PATCH_LTR=$(echo ${PATCH_NAME} | cut -c9) ;;
    * )
        print "${PATCH_NAME} is not a valid patch identifier."
        return 1 ;;
    esac

    # Validate the sub-fields syntax
    if [[  ${PATCH_VER} != [0-9][0-9][0-9] 
        || ${PATCH_PLATFORM} != [ASHLTP] 
        || ${HYPHEN} != "-" 
        || ${PATCH_FP} != [0-9][0-9]
        || (-n ${PATCH_LTR} && ${PATCH_LTR} != [A-Z]) ]]; then
        print "${PATCH_NAME} is not a valid patch identifier."
        return 1
    fi

    # Does the patch match the platform we're on?
    if [[ ${PATCH_PLATFORM} != ${PLATFORM_ID} ]]; then
        print "This ${PATCH_TYPE} cannot be installed on ${PLATFORM_NAME}."
        return 1
    fi
    
    # Relative path to patch id file
    ID_FILE=${PATCH_ID_DIR}/${PATCH_NAME}.txt

    # Initialize version specific defaults...
    case ${PATCH_VER} in
    321 )
        IDS_EXE="slapd"
        IDS_NAME="Secureway Directory Server v3.2.1" ;;
    322 )
        IDS_EXE="slapd"
        IDS_NAME="Secureway Directory Server v3.2.2" ;;
    410 )
        IDS_EXE="slapd"
        IDS_NAME="IBM Directory Server v4.1" ;;
    510 )
        IDS_EXE="ibmslapd"
        IDS_NAME="IBM Directory Server v5.1" ;;
    520 )
        IDS_EXE="ibmslapd"
        IDS_NAME="IBM Tivoli Directory Server v5.2" ;;
    * )
        print "${PATCH_VER} is not a known product version."
        return 1 ;;
    esac
    
    return 0
}

#####################################################################
#
# Chk_${OS}_Install
#
# Note: These should only be called from Chk_Install()
# 
# The following variable must be initialized first:
#   PATCH_VER   :: { 321 322 410 510 520 }
# and return 0 if that version client pkg. is installed, else !0
#

# Return 0 iff AIX client version $1 is installed 
Chk_AIX_Install()
{
    if [[ ${DEBUG} = TRUE ]]; then
        set -x
    fi
    
    case ${PATCH_VER} in
    321 )
        CHK_I_STR='3\.2\.1\.' ;;
    322 )
        CHK_I_STR='3\.2\.2\.' ;;
    410 )
        CHK_I_STR='4\.1\.0\.' ;;
    510 )
        CHK_I_STR='5\.1\.0\.' ;;
    520 )
        CHK_I_STR='5\.2\.0\.' ;;
    * )
        print "Unknown product version: ${PATCH_VER}"
        return 1 ;;
    esac

    lslpp -ql ldap.client.rte | grep "${CHK_I_STR}" >/dev/null
}

# Verify install path for SWD 3.2.2 on HP
Chk_HP_322_Install()
{
    while [[ ! -z ${LDAPHome} ]]; do
        if [[ ! -d ${LDAPHome} ]]; then
            print -u2 "\n${LDAPHome} is not a valid directory"
        elif [[ ! -a ${LDAPHome}/web/readme/buildno.txt ]]; then
            print -u2 "\n${IDS_NAME} not found in ${LDAPHome}"
        elif [[ ${LDAPHome} != /* ]]; then
            # qualify path
            cd ${LDAPHome}
            if [[ $? -ne 0 ]]; then
                print -u2 "\nUnable to qualify ${LDAPHome}"
                cd ${PATCH_DIR}
            else
                LDAPHome=$(pwd)
                cd ${PATCH_DIR}
                return 0
            fi
        else
            return 0
        fi
        read LDAPHome?"Please enter the correct install path: "
    done
    return 1
}

# Return 0 iff HP client version $1 is installed 
Chk_HP_Install()
{
    if [[ ${DEBUG} = TRUE ]]; then
        set -x
    fi
    
    case ${PATCH_VER} in
    # 321 ) There was no 3.2.1 for HP
    
    322 )
        IDS_EXE=""
        LDAPHome="/usr/ldapadt"
        Chk_HP_322_Install 
        return $? ;;
    410 )
        CHK_I_STR='IBM Directory 4\.1' ;;
    510 )
        CHK_I_STR='IBM Directory 5\.1' ;;
    520 )
        CHK_I_STR='IBM Tivoli Directory 5\.2' ;;
    * )
        print "Unknown product version: ${PATCH_VER}"
        return 1 ;;
    esac

    swlist | grep "${CHK_I_STR}" >/dev/null
}

# Return 0 iff Solaris client version $1 is installed 
Chk_Solaris_Install()
{
    if [[ ${DEBUG} = TRUE ]]; then
        set -x
    fi
    
    case ${PATCH_VER} in
    321 )
        CHK_I_STR='3.2.1.0' ;;
    322 )
        CHK_I_STR='3.2.2.0' ;;
    410 )
        CHK_I_STR='4.1.0.0' ;;
    510 )
        CHK_I_STR='5.1.0.0' ;;
    520 )
        CHK_I_STR='5.2.0.0' ;;
    * )
        print "Unknown product version: ${PATCH_VER}"
        return 1 ;;
    esac

    pkginfo -v ${CHK_I_STR} IBMldapc >/dev/null
}

# Return 0 iff Linux (L, P or T) client version $1 is installed 
Chk_Linux_Install()
{
    if [[ ${DEBUG} = TRUE ]]; then
        set -x
    fi
    
    case ${PATCH_VER} in
    321 )
        CHK_I_STR='ldap-client.*-3\.2-' ;;
    322 )
        CHK_I_STR='ldap-client.*-3\.2\.2-' ;;
    410 )
        CHK_I_STR='ldap-client.*-4\.1-' ;;
    510 )
        CHK_I_STR='ldap-client.*-5\.1-' ;;
    520 )
        CHK_I_STR='ldap-client.*-5\.2-' ;;
    * )
        print "Unknown product version: ${PATCH_VER}"
        return 1 ;;
    esac
    
    (rpm -qf /usr/ldap/bin/ldapsearch 2>&1) | grep "${CHK_I_STR}" >/dev/null
}


#####################################################################
#
# Chk_Install
#
# The following variables must be initialized first:
#   PLATFORM_ID in { A H L P S T }
#   PATCH_VER   in { 321 322 410 510 520 }
# and returns 0 if that version client pkg. is installed, else !0
#
Chk_Install()
{
    if [[ ${DEBUG} = TRUE ]]; then
        set -x
    fi
    
    case ${PLATFORM_ID} in
    A )
        Chk_AIX_Install ;;
    H )
        Chk_HP_Install ;;
    S )
        Chk_Solaris_Install ;;
    L )
        Chk_Linux_Install ;;
    P )
        Chk_Linux_Install ;;
    T )
        Chk_Linux_Install ;;
    * )
        print "Invalid platform ID: ${PLATFORM_ID}"
        return 1 ;;
    esac
}

#####################################################################
#
# Install_Update
#
# Install the patch or fix pack

RESTART_MSG=""  # Initialized in main()

Install_Update()
{
    if [[ ${DEBUG} = TRUE ]]; then
        set -x
    fi
    
    # Is there a patch to install?
    if [[ ! -a data/all_files.list ]]; then
       print "Error: can't find ${PATCH_DIR}/data/all_files.list"
       return 1
    fi

    # Create a list of the files to be replaced and their cksum's
    if [[ ! -a data/UNDO.tar.${COMP_EXT} ]]; then
        ( rm data/UNDO_files.* 2>&1 ) >/dev/null
        if [[ ! -a data/UNDO_files.list ]]; then
            for filename in $(cat data/all_files.list); do
                if [[ -f ${LDAPHome}/${filename} ]]; then
                    print ${filename} >>data/UNDO_files.list
                    cksum ${LDAPHome}/${filename} >>data/UNDO_files.sum
                fi
            done
        fi

        # Backup all the files to be replaced
        print "Backing up existing files..."
        cd ${LDAPHome}
        tar cf ${PATCH_DIR}/data/UNDO.tar $(cat ${PATCH_DIR}/data/UNDO_files.list)
        if [[ $? -ne 0 ]]; then
            # HACK - tar may return nonzero if userid/groupid's cannot be resolved, 
            # but archive is created anyway. See if this is the case...
            TAR_FAILED=TRUE
            if [[ -a ${PATCH_DIR}/data/UNDO.tar ]]; then
                LAST_FILE=$(tail -1 ${PATCH_DIR}/data/UNDO_files.list)
                if [[ -a ${LAST_FILE} ]]; then
                    cd ${PATCH_DIR}
                    mkdir tmp
                    cd tmp
                    tar cvf ../data/UNDO.tar ${LAST_FILE}
                    if cmp -s ${LDAPHome}/${LAST_FILE} ${LAST_FILE}; then
                        TAR_FAILED=FALSE
                    fi
                    cd ${PATCH_DIR}
                    rm -rf tmp
                fi
            fi
            
            if [[ ${TAR_FAILED} = TRUE ]]; then
                print "\nUnable to backup existing files."
                print "The ${PATCH_TYPE} will not be installed."
                return 1
            fi
        fi
        cd ${PATCH_DIR}
        ${COMPRESS} data/UNDO.tar
    fi

    # If there is a backup but no list, regenerate the list
    if [[ ! -a data/UNDO_files.list ]]; then
        # This should never happen, but just in case...
        print "Regenerating ${PATCH_TYPE} file list..."
        ${COMPRESS} -cd data/UNDO.tar.${COMP_EXT} | tar tf - >data/UNDO_files.list
        if [[ $? -ne 0 ]]; then
            print "\nUnable to generate filelist."
            print "the ${PATCH_TYPE} will not be installed."
            return 1
        fi
    fi

    print "Installing ${PATCH_TYPE} files..."

    # Run slibclean on AIX
    if [[ ${PLATFORM_ID} = A ]]; then
        slibclean
    fi

    # Remove tmp directory from any previously aborted install
    ( rm -rf tmp 2>&1 ) >/dev/null
    mkdir tmp
    cd tmp

    # Extract the files to be installed to a tmp directory
    ${COMPRESS} -cd ../data/patch.tar.${COMP_EXT} | tar xf - ${ID_FILE} $(cat ../data/UNDO_files.list)
    if [[ $? -ne 0 ]]; then
        print "\nError extracting ${PATCH_TYPE}"
        cd ..
        rm -rf tmp
        return 1
    fi

    # Copy files from tmp directory to ${LDAPHome}
    cp -r * ${LDAPHome}
    # If copy fails, restore backup files
    if [[ $? -ne 0 ]]; then
        print "The ${PATCH_TYPE} could not be installed."
        print "Restoring backup files..."
        cd ${LDAPHome}
        ( rm -f ${ID_FILE} 2>&1 ) >/dev/null
        ## Prompt users any errors if restoring fails
        ${COMPRESS} -cd ${PATCH_DIR}/data/UNDO.tar.${COMP_EXT} | tar xf -
        if [[ $? -ne 0 ]]; then
            print "Error restoring files."
            print "Please manually restore the backup files from ${PATCH_DIR}/data/UNDO.tar.${COMP_EXT}"
            return 1
        fi
        return 1
    fi

    # Remove tmp directory from successful install
    cd ${PATCH_DIR}
    rm -rf tmp

    # Generate cksum's for all newly installed files
    ( rm data/PATCH_files.* 2>&1 ) >/dev/null
    ## Capture sum and size for Signature file, and all the rest
    for filename in ${ID_FILE} $(cat data/UNDO_files.list); do
        cksum ${LDAPHome}/${filename} >>data/PATCH_files.sum
    done

    # Declare success
    print "The ${PATCH_TYPE} has been successfully installed."
    print ${RESTART_MSG}
    print "\n(Note: The patch can be uninstalled by running uninstall_update.sh"
    print "       only if data/UNDO.tar.${COMP_EXT} is not removed. )"

    return 0
}

#####################################################################
#
# Uninstall_Update
#
# Uninstall the patch or fix pack

Uninstall_Update()
{
    if [[ ${DEBUG} = TRUE ]]; then
        set -x
    fi
    
    # Is there a backup to restore from?
    if [[ ! -a data/UNDO.tar.${COMP_EXT} ]]; then
        print "${PATCH_DIR}/data/UNDO.tar.${COMP_EXT} cannot be found."
        print "The original files cannot be restored."
        return 1
    fi

    print "Uninstalling ${PATCH_TYPE} files ..."

    # Run slibclean on AIX
    if [[ ${PLATFORM_ID} = A ]]; then
        slibclean
    fi

    # Untar the backup directly over the ${LDAPHome} directory...
    cd ${LDAPHome}
    ( rm -f ${ID_FILE} 2>&1 ) >/dev/null
    ${COMPRESS} -cd ${PATCH_DIR}/data/UNDO.tar.${COMP_EXT} | tar xf -
    if [[ $? -ne 0 ]]; then
        print "IBM Directory Server could not be restored to it's previous version"
        print "Please manually restore the backup files from ${PATCH_DIR}/data/UNDO.tar.${COMP_EXT}"
    else
        print "IBM Directory Server has been restored to it's previous version."
        print ${RESTART_MSG}
    fi

    return 0
}

#####################################################################
#
# main
#
# Initialize script and either install or uninstall based on script name

    # Configure script based on patch directory name 
    Init_Patch_Data
    if [[ $? -ne 0 ]]; then
        return 1
    fi

    # Got root?
    _myid=$(id | cut -f 2 -d '(' | cut -f 1 -d ')')
    if [[ ${_myid} != "root" ]]; then
       print "You must be root to ${ACTION} this ${PATCH_TYPE}."
       return 1
    fi

    # Verify that base product is installed
    Chk_Install
    if [[ $? -ne 0 ]]; then
       print "This ${PATCH_TYPE} requires ${IDS_NAME}"
       return 1
    fi

    # If (un)installing the server, verify it's not running and format restart msg. 
    if [[ -n ${IDS_EXE} && -a /usr/bin/${IDS_EXE} ]]; then
        if [[ ${IDS_EXE} = "ibmslapd" ]]; then
            RESTART_MSG="You can restart ibmdiradm and ibmslapd."
            if ( ps -ef | grep ibmdiradm | grep -v grep ) ; then
               print "ibmdiradm must be stopped before ${ACTION}ing this ${PATCH_TYPE}."
               return 1
            fi
        else
            RESTART_MSG="You can restart slapd."
        fi

        if ( ps -ef | grep ${IDS_EXE} | grep -v grep ) ; then
           print "${IDS_EXE} must be stopped before ${ACTION}ing this ${PATCH_TYPE}."
           return 1
        fi
    fi

    # Finally, call Install or Uninstall...
    if [[ ${ACTION} = "install" ]]; then
        Install_Update
    else 
        Uninstall_Update
    fi
